(function (M0,M1,M2,M3,M4,M5,XX) {
/**
 * @preserve (C) 2000 - 2020 Nebular Streams - All Rights Reserved.
 */


(function(
    PROMISE,
    ObjectAssign,
    ObjectKeys,
    window_setTimeout,
    window_clearTimeout,
    window_location,
    window_addEventListener,
    DOC,
    BODY,
    HTML,
    HTMLCLASS,
    WINDOW,
    NIP,
    ERROR
) {

const
    D = false,
    Promise_resolve = PROMISE.resolve,
    Promise_reject = PROMISE.reject;

/* eslint-disable no-unused-vars,no-use-before-define */

function md5(input) {
	// FLOW: input -> utf8 input -> bin input -> bin md5 -> utf8 md5 -> hex md5
	var
		hex_vocab = "0123456789abcdef",

		raw_input = "",
		raw_input_bits_len,
		bin_input,
		bin_md5,
		raw_md5 = "",
		hex_md5 = "",

		i = -1,
		x,
		y;


	function safe_add(x, y) {
		var lsw = (x & 0xFFFF) + (y & 0xFFFF);
		return (((x >> 16) + (y >> 16) + (lsw >> 16)) << 16) | (lsw & 0xFFFF);
	}

	// basic operations
	function cmn(q, a, b, x, s, t) {
		var num = safe_add(safe_add(a, q), safe_add(x, t));
		return safe_add((num << s) | (num >>> (32 - s)), b);
	}

	function ff(a, b, c, d, x, s, t) {
		return cmn((b & c) | ((~b) & d), a, b, x, s, t);
	}

	function gg(a, b, c, d, x, s, t) {
		return cmn((b & d) | (c & (~d)), a, b, x, s, t);
	}

	function hh(a, b, c, d, x, s, t) {
		return cmn(b ^ c ^ d, a, b, x, s, t);
	}

	function ii(a, b, c, d, x, s, t) {
		return cmn(c ^ (b | (~d)), a, b, x, s, t);
	}


	// encode string to utf-8
	while (++i < input.length) {
		// decode utf-16 surrogate pairs
		x = input.charCodeAt(i);
		y = i + 1 < input.length ? input.charCodeAt(i + 1) : 0;
		if (0xD800 <= x && x <= 0xDBFF && 0xDC00 <= y && y <= 0xDFFF) {
			x = 0x10000 + ((x & 0x03FF) << 10) + (y & 0x03FF);
			i++;
		}

		// encode raw_input as utf-8
		if (x <= 0x7F) {
			raw_input += String.fromCharCode(x);
		} else if (x <= 0x7FF) {
			raw_input += String.fromCharCode(0xC0 | ((x >>> 6) & 0x1F),
				0x80 | (x & 0x3F));
		} else if (x <= 0xFFFF) {
			raw_input += String.fromCharCode(0xE0 | ((x >>> 12) & 0x0F),
				0x80 | ((x >>> 6) & 0x3F),
				0x80 | (x & 0x3F));
		} else if (x <= 0x1FFFFF) {
			raw_input += String.fromCharCode(0xF0 | ((x >>> 18) & 0x07),
				0x80 | ((x >>> 12) & 0x3F),
				0x80 | ((x >>> 6) & 0x3F),
				0x80 | (x & 0x3F));
		}
	}


	// number of bits in the raw utf-8 string
	raw_input_bits_len = raw_input.length * 8;


	// string to little-endian array words
	bin_input = new Array(raw_input.length >> 2);
	for (i = 0; i < bin_input.length; i++) {
		bin_input[i] = 0;
	}
	for (i = 0; i < raw_input_bits_len; i += 8) {
		bin_input[i >> 5] |= (raw_input.charCodeAt(i / 8) & 0xFF) << (i % 32);
	}


	// calculate md5 as little-endian array words
	// padding
	bin_input[raw_input_bits_len >> 5] |= 0x80 << ((raw_input_bits_len) % 32);
	bin_input[(((raw_input_bits_len + 64) >>> 9) << 4) + 14] = raw_input_bits_len;

	var
		a = 1732584193,
		b = -271733879,
		c = -1732584194,
		d = 271733878;

	for (i = 0; i < bin_input.length; i += 16) {
		var
			olda = a,
			oldb = b,
			oldc = c,
			oldd = d;

		a = ff(a, b, c, d, bin_input[i], 7, -680876936);
		d = ff(d, a, b, c, bin_input[i + 1], 12, -389564586);
		c = ff(c, d, a, b, bin_input[i + 2], 17, 606105819);
		b = ff(b, c, d, a, bin_input[i + 3], 22, -1044525330);
		a = ff(a, b, c, d, bin_input[i + 4], 7, -176418897);
		d = ff(d, a, b, c, bin_input[i + 5], 12, 1200080426);
		c = ff(c, d, a, b, bin_input[i + 6], 17, -1473231341);
		b = ff(b, c, d, a, bin_input[i + 7], 22, -45705983);
		a = ff(a, b, c, d, bin_input[i + 8], 7, 1770035416);
		d = ff(d, a, b, c, bin_input[i + 9], 12, -1958414417);
		c = ff(c, d, a, b, bin_input[i + 10], 17, -42063);
		b = ff(b, c, d, a, bin_input[i + 11], 22, -1990404162);
		a = ff(a, b, c, d, bin_input[i + 12], 7, 1804603682);
		d = ff(d, a, b, c, bin_input[i + 13], 12, -40341101);
		c = ff(c, d, a, b, bin_input[i + 14], 17, -1502002290);
		b = ff(b, c, d, a, bin_input[i + 15], 22, 1236535329);

		a = gg(a, b, c, d, bin_input[i + 1], 5, -165796510);
		d = gg(d, a, b, c, bin_input[i + 6], 9, -1069501632);
		c = gg(c, d, a, b, bin_input[i + 11], 14, 643717713);
		b = gg(b, c, d, a, bin_input[i], 20, -373897302);
		a = gg(a, b, c, d, bin_input[i + 5], 5, -701558691);
		d = gg(d, a, b, c, bin_input[i + 10], 9, 38016083);
		c = gg(c, d, a, b, bin_input[i + 15], 14, -660478335);
		b = gg(b, c, d, a, bin_input[i + 4], 20, -405537848);
		a = gg(a, b, c, d, bin_input[i + 9], 5, 568446438);
		d = gg(d, a, b, c, bin_input[i + 14], 9, -1019803690);
		c = gg(c, d, a, b, bin_input[i + 3], 14, -187363961);
		b = gg(b, c, d, a, bin_input[i + 8], 20, 1163531501);
		a = gg(a, b, c, d, bin_input[i + 13], 5, -1444681467);
		d = gg(d, a, b, c, bin_input[i + 2], 9, -51403784);
		c = gg(c, d, a, b, bin_input[i + 7], 14, 1735328473);
		b = gg(b, c, d, a, bin_input[i + 12], 20, -1926607734);

		a = hh(a, b, c, d, bin_input[i + 5], 4, -378558);
		d = hh(d, a, b, c, bin_input[i + 8], 11, -2022574463);
		c = hh(c, d, a, b, bin_input[i + 11], 16, 1839030562);
		b = hh(b, c, d, a, bin_input[i + 14], 23, -35309556);
		a = hh(a, b, c, d, bin_input[i + 1], 4, -1530992060);
		d = hh(d, a, b, c, bin_input[i + 4], 11, 1272893353);
		c = hh(c, d, a, b, bin_input[i + 7], 16, -155497632);
		b = hh(b, c, d, a, bin_input[i + 10], 23, -1094730640);
		a = hh(a, b, c, d, bin_input[i + 13], 4, 681279174);
		d = hh(d, a, b, c, bin_input[i], 11, -358537222);
		c = hh(c, d, a, b, bin_input[i + 3], 16, -722521979);
		b = hh(b, c, d, a, bin_input[i + 6], 23, 76029189);
		a = hh(a, b, c, d, bin_input[i + 9], 4, -640364487);
		d = hh(d, a, b, c, bin_input[i + 12], 11, -421815835);
		c = hh(c, d, a, b, bin_input[i + 15], 16, 530742520);
		b = hh(b, c, d, a, bin_input[i + 2], 23, -995338651);

		a = ii(a, b, c, d, bin_input[i], 6, -198630844);
		d = ii(d, a, b, c, bin_input[i + 7], 10, 1126891415);
		c = ii(c, d, a, b, bin_input[i + 14], 15, -1416354905);
		b = ii(b, c, d, a, bin_input[i + 5], 21, -57434055);
		a = ii(a, b, c, d, bin_input[i + 12], 6, 1700485571);
		d = ii(d, a, b, c, bin_input[i + 3], 10, -1894986606);
		c = ii(c, d, a, b, bin_input[i + 10], 15, -1051523);
		b = ii(b, c, d, a, bin_input[i + 1], 21, -2054922799);
		a = ii(a, b, c, d, bin_input[i + 8], 6, 1873313359);
		d = ii(d, a, b, c, bin_input[i + 15], 10, -30611744);
		c = ii(c, d, a, b, bin_input[i + 6], 15, -1560198380);
		b = ii(b, c, d, a, bin_input[i + 13], 21, 1309151649);
		a = ii(a, b, c, d, bin_input[i + 4], 6, -145523070);
		d = ii(d, a, b, c, bin_input[i + 11], 10, -1120210379);
		c = ii(c, d, a, b, bin_input[i + 2], 15, 718787259);
		b = ii(b, c, d, a, bin_input[i + 9], 21, -343485551);

		a = safe_add(a, olda);
		b = safe_add(b, oldb);
		c = safe_add(c, oldc);
		d = safe_add(d, oldd);
	}
	bin_md5 = [a, b, c, d];


	// little-endian array words to a string
	for (i = 0; i < bin_md5.length * 32; i += 8) {
		raw_md5 += String.fromCharCode((bin_md5[i >> 5] >>> (i % 32)) & 0xFF);
	}


	// convert the raw md5 string to a hex md5 string
	for (i = 0; i < raw_md5.length; i++) {
		x = raw_md5.charCodeAt(i);
		hex_md5 += hex_vocab.charAt((x >>> 4) & 0x0F) + hex_vocab.charAt(x & 0x0F);
	}

	return hex_md5;
}

function Requirer() {

	const globals = typeof window === "undefined" ? global : window;
	if  (typeof globals.require === "function") return globals.require;

	const
		modules = {},
		cache = {},
		aliases = {},
		has = ({}).hasOwnProperty;


	// eslint-disable-next-line no-underscore-dangle
	const _cmp = "components/";

	function unalias(alias, loaderPath) {
		let start = 0;
		if (loaderPath) {
			if (loaderPath.indexOf(_cmp) === 0) {
				start = _cmp.length;
			}
			if (loaderPath.indexOf("/", start) > 0) {
				loaderPath = loaderPath.substring(start, loaderPath.indexOf("/", start));
			}
		}

		const result = aliases[alias + "/index.js"] || aliases[loaderPath + "/deps/" + alias + "/index.js"];
		if (result) {
			return _cmp + result.substring(0, result.length - ".js".length);
		}
		return alias;
	}

	// eslint-disable-next-line no-underscore-dangle
	const _reg = /^\.\.?(\/|$)/;

	function expand(root, name) {

		let part;

		const
			results = [],
			parts = (_reg.test(name) ? root + "/" + name : name).split("/");

		for (let i = 0, length = parts.length; i < length; i++) {
			part = parts[i];
			if (part === "..") {
				results.pop();
			} else if (part !== "." && part !== "") {
				results.push(part);
			}
		}
		return results.join("/");
	}

	function dirname(path) {
		return path.split("/")
			.slice(0, -1)
			.join("/");
	}

	function requireInternal(name, loaderPath) {

		if (loaderPath == null) loaderPath = "/";
		const path = unalias(name, loaderPath);

		if (has.call(cache, path)) return cache[path].exports;
		if (has.call(modules, path)) return initModule(path, modules[path]);


		const namemd5 = md5(name);
		const pathmd5 = unalias(namemd5, loaderPath);

		if (has.call(cache, pathmd5)) return cache[pathmd5].exports;
		if (has.call(modules, pathmd5)) return initModule(pathmd5, modules[pathmd5]);


		let dirIndex = expand(path, "./index");
		if (has.call(cache, dirIndex)) return cache[dirIndex].exports;
		if (has.call(modules, dirIndex)) return initModule(dirIndex, modules[dirIndex]);


		dirIndex = expand(pathmd5, "./index");
		if (has.call(cache, dirIndex)) return cache[dirIndex].exports;
		if (has.call(modules, dirIndex)) return initModule(dirIndex, modules[dirIndex]);

		throw new Error("Cannot find module \"" + name + "\" from " + "\"" + loaderPath + "\"");
	}

	function localRequire(path) {
		return function expanded(name) {
			const absolute = expand(dirname(path), name);
			return requireInternal(absolute, path);
		};
	}

	function initModule(name, definition) {

		const module = {
			id: name,
			exports: {}
		};

		cache[name] = module;
		definition(module.exports, localRequire(name), module);
		return module.exports;

	}


	requireInternal.alias = function (from, to) {
		aliases[to] = from;
	};

	requireInternal.register = requireInternal.define = function (bundle, fn) {
		if (typeof bundle === "object") {
			for (const key in bundle) {
				if (has.call(bundle, key)) {
					modules[key] = bundle[key];
				}
			}
		} else {
			modules[bundle] = fn;
		}
	};

	requireInternal.list = function () {
		const result = [];
		for (const item in modules) {
			if (has.call(modules, item)) {
				result.push(item);
			}
		}
		return result;
	};

	requireInternal.brunch = true;
	requireInternal._cache = cache;

	globals.require = requireInternal;

	return requireInternal;
}

const
	require = Requirer(),
	requireRegister = require.register,
	requireDefine = require.define;
requireRegister(M0, function (exports, require, module) {
if (typeof Element.prototype.clearChildren === "undefined") {
	Object.defineProperty(Element.prototype, "clearChildren", {
		configurable: true,
		enumerable: false,
		value: () => {
			while (this.firstChild) this.removeChild(this.lastChild);
		}
	});
}

});
requireRegister(M1, function (exports, require, module) {
/*
CryptoJS v3.0.2
code.google.com/p/crypto-js
(c) 2009-2012 by Jeff Mott. All rights reserved.
code.google.com/p/crypto-js/wiki/License
*/
var CryptoJS = CryptoJS || function (p, h) {
	var i = {},
		l = i.lib = {},
		r = l.Base = function () {
			function a() {
			}

			return {
				extend: function (e) {
					a.prototype = this;
					var c = new a;
					e && c.mixIn(e);
					c.$super = this;
					return c;
				},
				create: function () {
					var a = this.extend();
					a.init.apply(a, arguments);
					return a;
				},
				init: function () {
				},
				mixIn: function (a) {
					for (var c in a) a.hasOwnProperty(c) && (this[c] = a[c]);
					a.hasOwnProperty("toString") && (this.toString = a.toString);
				},
				clone: function () {
					return this.$super.extend(this);
				}
			};
		}(),
		o = l.WordArray = r.extend({
			init: function (a, e) {
				a =
					this.words = a || [];
				this.sigBytes = e != h ? e : 4 * a.length;
			},
			toString: function (a) {
				return (a || s).stringify(this);
			},
			concat: function (a) {
				var e = this.words,
					c = a.words,
					b = this.sigBytes,
					a = a.sigBytes;
				this.clamp();
				if (b % 4) for (var d = 0; d < a; d++) e[b + d >>> 2] |= (c[d >>> 2] >>> 24 - 8 * (d % 4) & 255) << 24 - 8 * ((b + d) % 4); else if (65535 < c.length) for (d = 0; d < a; d += 4) e[b + d >>> 2] = c[d >>> 2]; else e.push.apply(e, c);
				this.sigBytes += a;
				return this;
			},
			clamp: function () {
				var a = this.words,
					e = this.sigBytes;
				a[e >>> 2] &= 4294967295 << 32 - 8 * (e % 4);
				a.length = p.ceil(e / 4);
			},
			clone: function () {
				var a =
					r.clone.call(this);
				a.words = this.words.slice(0);
				return a;
			},
			random: function (a) {
				for (var e = [], c = 0; c < a; c += 4) e.push(4294967296 * p.random() | 0);
				return o.create(e, a);
			}
		}),
		m = i.enc = {},
		s = m.Hex = {
			stringify: function (a) {
				for (var e = a.words, a = a.sigBytes, c = [], b = 0; b < a; b++) {
					var d = e[b >>> 2] >>> 24 - 8 * (b % 4) & 255;
					c.push((d >>> 4).toString(16));
					c.push((d & 15).toString(16));
				}
				return c.join("");
			},
			parse: function (a) {
				for (var e = a.length, c = [], b = 0; b < e; b += 2) c[b >>> 3] |= parseInt(a.substr(b, 2), 16) << 24 - 4 * (b % 8);
				return o.create(c, e / 2);
			}
		},
		n = m.Latin1 = {
			stringify: function (a) {
				for (var e =
					a.words, a = a.sigBytes, c = [], b = 0; b < a; b++) {
					c.push(String.fromCharCode(e[b >>> 2] >>> 24 - 8 * (b % 4) & 255));
				}
				return c.join("");
			},
			parse: function (a) {
				for (var e = a.length, c = [], b = 0; b < e; b++) c[b >>> 2] |= (a.charCodeAt(b) & 255) << 24 - 8 * (b % 4);
				return o.create(c, e);
			}
		},
		k = m.Utf8 = {
			stringify: function (a) {
				try {
					return decodeURIComponent(escape(n.stringify(a)));
				} catch (e) {
					throw Error("Malformed UTF-8 data");
				}
			},
			parse: function (a) {
				return n.parse(unescape(encodeURIComponent(a)));
			}
		},
		f = l.BufferedBlockAlgorithm = r.extend({
			reset: function () {
				this._data = o.create();
				this._nDataBytes = 0;
			},
			_append: function (a) {
				"string" == typeof a && (a = k.parse(a));
				this._data.concat(a);
				this._nDataBytes += a.sigBytes;
			},
			_process: function (a) {
				var e = this._data,
					c = e.words,
					b = e.sigBytes,
					d = this.blockSize,
					q = b / (4 * d),
					q = a ? p.ceil(q) : p.max((q | 0) - this._minBufferSize, 0),
					a = q * d,
					b = p.min(4 * a, b);
				if (a) {
					for (var j = 0; j < a; j += d) this._doProcessBlock(c, j);
					j = c.splice(0, a);
					e.sigBytes -= b;
				}
				return o.create(j, b);
			},
			clone: function () {
				var a = r.clone.call(this);
				a._data = this._data.clone();
				return a;
			},
			_minBufferSize: 0
		});
	l.Hasher = f.extend({
		init: function () {
			this.reset();
		},
		reset: function () {
			f.reset.call(this);
			this._doReset();
		},
		update: function (a) {
			this._append(a);
			this._process();
			return this;
		},
		finalize: function (a) {
			a && this._append(a);
			this._doFinalize();
			return this._hash;
		},
		clone: function () {
			var a = f.clone.call(this);
			a._hash = this._hash.clone();
			return a;
		},
		blockSize: 16,
		_createHelper: function (a) {
			return function (e, c) {
				return a.create(c)
					.finalize(e);
			};
		},
		_createHmacHelper: function (a) {
			return function (e, c) {
				return g.HMAC.create(a, c)
					.finalize(e);
			};
		}
	});
	var g = i.algo = {};
	return i;
}(Math);
(function () {
	var p = CryptoJS,
		h = p.lib.WordArray;
	p.enc.Base64 = {
		stringify: function (i) {
			var l = i.words,
				h = i.sigBytes,
				o = this._map;
			i.clamp();
			for (var i = [], m = 0; m < h; m += 3) for (var s = (l[m >>> 2] >>> 24 - 8 * (m % 4) & 255) << 16 | (l[m + 1 >>> 2] >>> 24 - 8 * ((m + 1) % 4) & 255) << 8 | l[m + 2 >>> 2] >>> 24 - 8 * ((m + 2) % 4) & 255, n = 0; 4 > n && m + 0.75 * n < h; n++) i.push(o.charAt(s >>> 6 * (3 - n) & 63));
			if (l = o.charAt(64)) for (; i.length % 4;) i.push(l);
			return i.join("");
		},
		parse: function (i) {
			var i = i.replace(/\s/g, ""),
				l = i.length,
				r = this._map,
				o = r.charAt(64);
			o && (o = i.indexOf(o), -1 != o && (l = o));
			for (var o = [], m = 0, s = 0; s < l; s++) {
				if (s % 4) {
					var n = r.indexOf(i.charAt(s - 1)) << 2 * (s % 4),
						k = r.indexOf(i.charAt(s)) >>> 6 - 2 * (s % 4);
					o[m >>> 2] |= (n | k) << 24 - 8 * (m % 4);
					m++;
				}
			}
			return h.create(o, m);
		},
		_map: "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="
	};
})();
(function (p) {
	function h(f, g, a, e, c, b, d) {
		f = f + (g & a | ~g & e) + c + d;
		return (f << b | f >>> 32 - b) + g;
	}

	function i(f, g, a, e, c, b, d) {
		f = f + (g & e | a & ~e) + c + d;
		return (f << b | f >>> 32 - b) + g;
	}

	function l(f, g, a, e, c, b, d) {
		f = f + (g ^ a ^ e) + c + d;
		return (f << b | f >>> 32 - b) + g;
	}

	function r(f, g, a, e, c, b, d) {
		f = f + (a ^ (g | ~e)) + c + d;
		return (f << b | f >>> 32 - b) + g;
	}

	var o = CryptoJS,
		m = o.lib,
		s = m.WordArray,
		m = m.Hasher,
		n = o.algo,
		k = [];
	(function () {
		for (var f = 0; 64 > f; f++) k[f] = 4294967296 * p.abs(p.sin(f + 1)) | 0;
	})();
	n = n.MD5 = m.extend({
		_doReset: function () {
			this._hash = s.create([1732584193, 4023233417,
				2562383102, 271733878]);
		},
		_doProcessBlock: function (f, g) {
			for (var a = 0; 16 > a; a++) {
				var e = g + a,
					c = f[e];
				f[e] = (c << 8 | c >>> 24) & 16711935 | (c << 24 | c >>> 8) & 4278255360;
			}
			for (var e = this._hash.words, c = e[0], b = e[1], d = e[2], q = e[3], a = 0; 64 > a; a += 4) {
				16 > a ? (c = h(c, b, d, q, f[g + a], 7, k[a]), q = h(q, c, b, d, f[g + a + 1], 12, k[a + 1]), d = h(d, q, c, b, f[g + a + 2], 17, k[a + 2]), b = h(b, d, q, c, f[g + a + 3], 22, k[a + 3])) : 32 > a ? (c = i(c, b, d, q, f[g + (a + 1) % 16], 5, k[a]), q = i(q, c, b, d, f[g + (a + 6) % 16], 9, k[a + 1]), d = i(d, q, c, b, f[g + (a + 11) % 16], 14, k[a + 2]), b = i(b, d, q, c, f[g + a % 16], 20, k[a + 3])) : 48 > a ? (c =
					l(c, b, d, q, f[g + (3 * a + 5) % 16], 4, k[a]), q = l(q, c, b, d, f[g + (3 * a + 8) % 16], 11, k[a + 1]), d = l(d, q, c, b, f[g + (3 * a + 11) % 16], 16, k[a + 2]), b = l(b, d, q, c, f[g + (3 * a + 14) % 16], 23, k[a + 3])) : (c = r(c, b, d, q, f[g + 3 * a % 16], 6, k[a]), q = r(q, c, b, d, f[g + (3 * a + 7) % 16], 10, k[a + 1]), d = r(d, q, c, b, f[g + (3 * a + 14) % 16], 15, k[a + 2]), b = r(b, d, q, c, f[g + (3 * a + 5) % 16], 21, k[a + 3]));
			}
			e[0] = e[0] + c | 0;
			e[1] = e[1] + b | 0;
			e[2] = e[2] + d | 0;
			e[3] = e[3] + q | 0;
		},
		_doFinalize: function () {
			var f = this._data,
				g = f.words,
				a = 8 * this._nDataBytes,
				e = 8 * f.sigBytes;
			g[e >>> 5] |= 128 << 24 - e % 32;
			g[(e + 64 >>> 9 << 4) + 14] = (a << 8 | a >>>
				24) & 16711935 | (a << 24 | a >>> 8) & 4278255360;
			f.sigBytes = 4 * (g.length + 1);
			this._process();
			f = this._hash.words;
			for (g = 0; 4 > g; g++) a = f[g], f[g] = (a << 8 | a >>> 24) & 16711935 | (a << 24 | a >>> 8) & 4278255360;
		}
	});
	o.MD5 = m._createHelper(n);
	o.HmacMD5 = m._createHmacHelper(n);
})(Math);
(function () {
	var p = CryptoJS,
		h = p.lib,
		i = h.Base,
		l = h.WordArray,
		h = p.algo,
		r = h.EvpKDF = i.extend({
			cfg: i.extend({
				keySize: 4,
				hasher: h.MD5,
				iterations: 1
			}),
			init: function (i) {
				this.cfg = this.cfg.extend(i);
			},
			compute: function (i, m) {
				for (var h = this.cfg, n = h.hasher.create(), k = l.create(), f = k.words, g = h.keySize, h = h.iterations; f.length < g;) {
					a && n.update(a);
					var a = n.update(i)
						.finalize(m);
					n.reset();
					for (var e = 1; e < h; e++) a = n.finalize(a), n.reset();
					k.concat(a);
				}
				k.sigBytes = 4 * g;
				return k;
			}
		});
	p.EvpKDF = function (i, l, h) {
		return r.create(h)
			.compute(i,
				l);
	};
})();
CryptoJS.lib.Cipher || function (p) {
	var h = CryptoJS,
		i = h.lib,
		l = i.Base,
		r = i.WordArray,
		o = i.BufferedBlockAlgorithm,
		m = h.enc.Base64,
		s = h.algo.EvpKDF,
		n = i.Cipher = o.extend({
			cfg: l.extend(),
			createEncryptor: function (b, d) {
				return this.create(this._ENC_XFORM_MODE, b, d);
			},
			createDecryptor: function (b, d) {
				return this.create(this._DEC_XFORM_MODE, b, d);
			},
			init: function (b, d, a) {
				this.cfg = this.cfg.extend(a);
				this._xformMode = b;
				this._key = d;
				this.reset();
			},
			reset: function () {
				o.reset.call(this);
				this._doReset();
			},
			process: function (b) {
				this._append(b);
				return this._process();
			},
			finalize: function (b) {
				b && this._append(b);
				return this._doFinalize();
			},
			keySize: 4,
			ivSize: 4,
			_ENC_XFORM_MODE: 1,
			_DEC_XFORM_MODE: 2,
			_createHelper: function () {
				return function (b) {
					return {
						encrypt: function (a, q, j) {
							return ("string" == typeof q ? c : e).encrypt(b, a, q, j);
						},
						decrypt: function (a, q, j) {
							return ("string" == typeof q ? c : e).decrypt(b, a, q, j);
						}
					};
				};
			}()
		});
	i.StreamCipher = n.extend({
		_doFinalize: function () {
			return this._process(!0);
		},
		blockSize: 1
	});
	var k = h.mode = {},
		f = i.BlockCipherMode = l.extend({
			createEncryptor: function (b, a) {
				return this.Encryptor.create(b,
					a);
			},
			createDecryptor: function (b, a) {
				return this.Decryptor.create(b, a);
			},
			init: function (b, a) {
				this._cipher = b;
				this._iv = a;
			}
		}),
		k = k.CBC = function () {
			function b(b, a, d) {
				var c = this._iv;
				c ? this._iv = p : c = this._prevBlock;
				for (var e = 0; e < d; e++) b[a + e] ^= c[e];
			}

			var a = f.extend();
			a.Encryptor = a.extend({
				processBlock: function (a, d) {
					var c = this._cipher,
						e = c.blockSize;
					b.call(this, a, d, e);
					c.encryptBlock(a, d);
					this._prevBlock = a.slice(d, d + e);
				}
			});
			a.Decryptor = a.extend({
				processBlock: function (a, d) {
					var c = this._cipher,
						e = c.blockSize,
						f = a.slice(d, d +
							e);
					c.decryptBlock(a, d);
					b.call(this, a, d, e);
					this._prevBlock = f;
				}
			});
			return a;
		}(),
		g = (h.pad = {}).Pkcs7 = {
			pad: function (b, a) {
				for (var c = 4 * a, c = c - b.sigBytes % c, e = c << 24 | c << 16 | c << 8 | c, f = [], g = 0; g < c; g += 4) f.push(e);
				c = r.create(f, c);
				b.concat(c);
			},
			unpad: function (b) {
				b.sigBytes -= b.words[b.sigBytes - 1 >>> 2] & 255;
			}
		};
	i.BlockCipher = n.extend({
		cfg: n.cfg.extend({
			mode: k,
			padding: g
		}),
		reset: function () {
			n.reset.call(this);
			var b = this.cfg,
				a = b.iv,
				b = b.mode;
			if (this._xformMode == this._ENC_XFORM_MODE) {
				var c = b.createEncryptor;
			} else {
				c = b.createDecryptor,
					this._minBufferSize = 1;
			}
			this._mode = c.call(b, this, a && a.words);
		},
		_doProcessBlock: function (b, a) {
			this._mode.processBlock(b, a);
		},
		_doFinalize: function () {
			var b = this.cfg.padding;
			if (this._xformMode == this._ENC_XFORM_MODE) {
				b.pad(this._data, this.blockSize);
				var a = this._process(!0);
			} else {
				a = this._process(!0), b.unpad(a);
			}
			return a;
		},
		blockSize: 4
	});
	var a = i.CipherParams = l.extend({
			init: function (a) {
				this.mixIn(a);
			},
			toString: function (a) {
				return (a || this.formatter).stringify(this);
			}
		}),
		k = (h.format = {}).OpenSSL = {
			stringify: function (a) {
				var d =
						a.ciphertext,
					a = a.salt,
					d = (a ? r.create([1398893684, 1701076831])
						.concat(a)
						.concat(d) : d).toString(m);
				return d = d.replace(/(.{64})/g, "$1\n");
			},
			parse: function (b) {
				var b = m.parse(b),
					d = b.words;
				if (1398893684 == d[0] && 1701076831 == d[1]) {
					var c = r.create(d.slice(2, 4));
					d.splice(0, 4);
					b.sigBytes -= 16;
				}
				return a.create({
					ciphertext: b,
					salt: c
				});
			}
		},
		e = i.SerializableCipher = l.extend({
			cfg: l.extend({format: k}),
			encrypt: function (b, d, c, e) {
				var e = this.cfg.extend(e),
					f = b.createEncryptor(c, e),
					d = f.finalize(d),
					f = f.cfg;
				return a.create({
					ciphertext: d,
					key: c,
					iv: f.iv,
					algorithm: b,
					mode: f.mode,
					padding: f.padding,
					blockSize: b.blockSize,
					formatter: e.format
				});
			},
			decrypt: function (a, c, e, f) {
				f = this.cfg.extend(f);
				c = this._parse(c, f.format);
				return a.createDecryptor(e, f)
					.finalize(c.ciphertext);
			},
			_parse: function (a, c) {
				return "string" == typeof a ? c.parse(a) : a;
			}
		}),
		h = (h.kdf = {}).OpenSSL = {
			compute: function (b, c, e, f) {
				f || (f = r.random(8));
				b = s.create({keySize: c + e})
					.compute(b, f);
				e = r.create(b.words.slice(c), 4 * e);
				b.sigBytes = 4 * c;
				return a.create({
					key: b,
					iv: e,
					salt: f
				});
			}
		},
		c = i.PasswordBasedCipher =
			e.extend({
				cfg: e.cfg.extend({kdf: h}),
				encrypt: function (a, c, f, j) {
					j = this.cfg.extend(j);
					f = j.kdf.compute(f, a.keySize, a.ivSize);
					j.iv = f.iv;
					a = e.encrypt.call(this, a, c, f.key, j);
					a.mixIn(f);
					return a;
				},
				decrypt: function (a, c, f, j) {
					j = this.cfg.extend(j);
					c = this._parse(c, j.format);
					f = j.kdf.compute(f, a.keySize, a.ivSize, c.salt);
					j.iv = f.iv;
					return e.decrypt.call(this, a, c, f.key, j);
				}
			});
}();
(function () {
	var p = CryptoJS,
		h = p.lib.BlockCipher,
		i = p.algo,
		l = [],
		r = [],
		o = [],
		m = [],
		s = [],
		n = [],
		k = [],
		f = [],
		g = [],
		a = [];
	(function () {
		for (var c = [], b = 0; 256 > b; b++) c[b] = 128 > b ? b << 1 : b << 1 ^ 283;
		for (var d = 0, e = 0, b = 0; 256 > b; b++) {
			var j = e ^ e << 1 ^ e << 2 ^ e << 3 ^ e << 4,
				j = j >>> 8 ^ j & 255 ^ 99;
			l[d] = j;
			r[j] = d;
			var i = c[d],
				h = c[i],
				p = c[h],
				t = 257 * c[j] ^ 16843008 * j;
			o[d] = t << 24 | t >>> 8;
			m[d] = t << 16 | t >>> 16;
			s[d] = t << 8 | t >>> 24;
			n[d] = t;
			t = 16843009 * p ^ 65537 * h ^ 257 * i ^ 16843008 * d;
			k[j] = t << 24 | t >>> 8;
			f[j] = t << 16 | t >>> 16;
			g[j] = t << 8 | t >>> 24;
			a[j] = t;
			d ? (d = i ^ c[c[c[p ^ i]]], e ^= c[c[e]]) : d = e = 1;
		}
	})();
	var e = [0, 1, 2, 4, 8, 16, 32, 64, 128, 27, 54],
		i = i.AES = h.extend({
			_doReset: function () {
				for (var c = this._key, b = c.words, d = c.sigBytes / 4, c = 4 * ((this._nRounds = d + 6) + 1), i = this._keySchedule = [], j = 0; j < c; j++) {
					if (j < d) {
						i[j] = b[j];
					} else {
						var h = i[j - 1];
						j % d ? 6 < d && 4 == j % d && (h = l[h >>> 24] << 24 | l[h >>> 16 & 255] << 16 | l[h >>> 8 & 255] << 8 | l[h & 255]) : (h = h << 8 | h >>> 24, h = l[h >>> 24] << 24 | l[h >>> 16 & 255] << 16 | l[h >>> 8 & 255] << 8 | l[h & 255], h ^= e[j / d | 0] << 24);
						i[j] = i[j - d] ^ h;
					}
				}
				b = this._invKeySchedule = [];
				for (d = 0; d < c; d++) {
					j = c - d, h = d % 4 ? i[j] : i[j - 4], b[d] = 4 > d || 4 >= j ? h : k[l[h >>> 24]] ^ f[l[h >>>
					16 & 255]] ^ g[l[h >>> 8 & 255]] ^ a[l[h & 255]];
				}
			},
			encryptBlock: function (a, b) {
				this._doCryptBlock(a, b, this._keySchedule, o, m, s, n, l);
			},
			decryptBlock: function (c, b) {
				var d = c[b + 1];
				c[b + 1] = c[b + 3];
				c[b + 3] = d;
				this._doCryptBlock(c, b, this._invKeySchedule, k, f, g, a, r);
				d = c[b + 1];
				c[b + 1] = c[b + 3];
				c[b + 3] = d;
			},
			_doCryptBlock: function (a, b, d, e, f, h, i, g) {
				for (var l = this._nRounds, k = a[b] ^ d[0], m = a[b + 1] ^ d[1], o = a[b + 2] ^ d[2], n = a[b + 3] ^ d[3], p = 4, r = 1; r < l; r++) {
					var s = e[k >>> 24] ^ f[m >>> 16 & 255] ^ h[o >>> 8 & 255] ^ i[n & 255] ^ d[p++],
						u = e[m >>> 24] ^ f[o >>> 16 & 255] ^ h[n >>> 8 & 255] ^
							i[k & 255] ^ d[p++],
						v = e[o >>> 24] ^ f[n >>> 16 & 255] ^ h[k >>> 8 & 255] ^ i[m & 255] ^ d[p++],
						n = e[n >>> 24] ^ f[k >>> 16 & 255] ^ h[m >>> 8 & 255] ^ i[o & 255] ^ d[p++],
						k = s,
						m = u,
						o = v;
				}
				s = (g[k >>> 24] << 24 | g[m >>> 16 & 255] << 16 | g[o >>> 8 & 255] << 8 | g[n & 255]) ^ d[p++];
				u = (g[m >>> 24] << 24 | g[o >>> 16 & 255] << 16 | g[n >>> 8 & 255] << 8 | g[k & 255]) ^ d[p++];
				v = (g[o >>> 24] << 24 | g[n >>> 16 & 255] << 16 | g[k >>> 8 & 255] << 8 | g[m & 255]) ^ d[p++];
				n = (g[n >>> 24] << 24 | g[k >>> 16 & 255] << 16 | g[m >>> 8 & 255] << 8 | g[o & 255]) ^ d[p++];
				a[b] = s;
				a[b + 1] = u;
				a[b + 2] = v;
				a[b + 3] = n;
			},
			keySize: 8
		});
	p.AES = h._createHelper(i);
})();


/*
CryptoJS v3.0.2
code.google.com/p/crypto-js
(c) 2009-2012 by Jeff Mott. All rights reserved.
code.google.com/p/crypto-js/wiki/License
*/
var CryptoJS = CryptoJS || function (g, i) {
	var f = {},
		b = f.lib = {},
		m = b.Base = function () {
			function a() {
			}

			return {
				extend: function (e) {
					a.prototype = this;
					var c = new a;
					e && c.mixIn(e);
					c.$super = this;
					return c;
				},
				create: function () {
					var a = this.extend();
					a.init.apply(a, arguments);
					return a;
				},
				init: function () {
				},
				mixIn: function (a) {
					for (var c in a) a.hasOwnProperty(c) && (this[c] = a[c]);
					a.hasOwnProperty("toString") && (this.toString = a.toString);
				},
				clone: function () {
					return this.$super.extend(this);
				}
			};
		}(),
		l = b.WordArray = m.extend({
			init: function (a, e) {
				a =
					this.words = a || [];
				this.sigBytes = e != i ? e : 4 * a.length;
			},
			toString: function (a) {
				return (a || d).stringify(this);
			},
			concat: function (a) {
				var e = this.words,
					c = a.words,
					o = this.sigBytes,
					a = a.sigBytes;
				this.clamp();
				if (o % 4) for (var b = 0; b < a; b++) e[o + b >>> 2] |= (c[b >>> 2] >>> 24 - 8 * (b % 4) & 255) << 24 - 8 * ((o + b) % 4); else if (65535 < c.length) for (b = 0; b < a; b += 4) e[o + b >>> 2] = c[b >>> 2]; else e.push.apply(e, c);
				this.sigBytes += a;
				return this;
			},
			clamp: function () {
				var a = this.words,
					e = this.sigBytes;
				a[e >>> 2] &= 4294967295 << 32 - 8 * (e % 4);
				a.length = g.ceil(e / 4);
			},
			clone: function () {
				var a =
					m.clone.call(this);
				a.words = this.words.slice(0);
				return a;
			},
			random: function (a) {
				for (var e = [], c = 0; c < a; c += 4) e.push(4294967296 * g.random() | 0);
				return l.create(e, a);
			}
		}),
		n = f.enc = {},
		d = n.Hex = {
			stringify: function (a) {
				for (var e = a.words, a = a.sigBytes, c = [], b = 0; b < a; b++) {
					var d = e[b >>> 2] >>> 24 - 8 * (b % 4) & 255;
					c.push((d >>> 4).toString(16));
					c.push((d & 15).toString(16));
				}
				return c.join("");
			},
			parse: function (a) {
				for (var e = a.length, c = [], b = 0; b < e; b += 2) c[b >>> 3] |= parseInt(a.substr(b, 2), 16) << 24 - 4 * (b % 8);
				return l.create(c, e / 2);
			}
		},
		j = n.Latin1 = {
			stringify: function (a) {
				for (var e =
					a.words, a = a.sigBytes, b = [], d = 0; d < a; d++) {
					b.push(String.fromCharCode(e[d >>> 2] >>> 24 - 8 * (d % 4) & 255));
				}
				return b.join("");
			},
			parse: function (a) {
				for (var b = a.length, c = [], d = 0; d < b; d++) c[d >>> 2] |= (a.charCodeAt(d) & 255) << 24 - 8 * (d % 4);
				return l.create(c, b);
			}
		},
		k = n.Utf8 = {
			stringify: function (a) {
				try {
					return decodeURIComponent(escape(j.stringify(a)));
				} catch (b) {
					throw Error("Malformed UTF-8 data");
				}
			},
			parse: function (a) {
				return j.parse(unescape(encodeURIComponent(a)));
			}
		},
		h = b.BufferedBlockAlgorithm = m.extend({
			reset: function () {
				this._data = l.create();
				this._nDataBytes = 0;
			},
			_append: function (a) {
				"string" == typeof a && (a = k.parse(a));
				this._data.concat(a);
				this._nDataBytes += a.sigBytes;
			},
			_process: function (a) {
				var b = this._data,
					c = b.words,
					d = b.sigBytes,
					j = this.blockSize,
					h = d / (4 * j),
					h = a ? g.ceil(h) : g.max((h | 0) - this._minBufferSize, 0),
					a = h * j,
					d = g.min(4 * a, d);
				if (a) {
					for (var f = 0; f < a; f += j) this._doProcessBlock(c, f);
					f = c.splice(0, a);
					b.sigBytes -= d;
				}
				return l.create(f, d);
			},
			clone: function () {
				var a = m.clone.call(this);
				a._data = this._data.clone();
				return a;
			},
			_minBufferSize: 0
		});
	b.Hasher = h.extend({
		init: function () {
			this.reset();
		},
		reset: function () {
			h.reset.call(this);
			this._doReset();
		},
		update: function (a) {
			this._append(a);
			this._process();
			return this;
		},
		finalize: function (a) {
			a && this._append(a);
			this._doFinalize();
			return this._hash;
		},
		clone: function () {
			var a = h.clone.call(this);
			a._hash = this._hash.clone();
			return a;
		},
		blockSize: 16,
		_createHelper: function (a) {
			return function (b, c) {
				return a.create(c)
					.finalize(b);
			};
		},
		_createHmacHelper: function (a) {
			return function (b, c) {
				return u.HMAC.create(a, c)
					.finalize(b);
			};
		}
	});
	var u = f.algo = {};
	return f;
}(Math);
(function () {
	var g = CryptoJS,
		i = g.lib,
		f = i.WordArray,
		i = i.Hasher,
		b = [],
		m = g.algo.SHA1 = i.extend({
			_doReset: function () {
				this._hash = f.create([1732584193, 4023233417, 2562383102, 271733878, 3285377520]);
			},
			_doProcessBlock: function (f, n) {
				for (var d = this._hash.words, j = d[0], k = d[1], h = d[2], g = d[3], a = d[4], e = 0; 80 > e; e++) {
					if (16 > e) {
						b[e] = f[n + e] | 0;
					} else {
						var c = b[e - 3] ^ b[e - 8] ^ b[e - 14] ^ b[e - 16];
						b[e] = c << 1 | c >>> 31;
					}
					c = (j << 5 | j >>> 27) + a + b[e];
					c = 20 > e ? c + ((k & h | ~k & g) + 1518500249) : 40 > e ? c + ((k ^ h ^ g) + 1859775393) : 60 > e ? c + ((k & h | k & g | h & g) - 1894007588) : c + ((k ^ h ^ g) -
						899497514);
					a = g;
					g = h;
					h = k << 30 | k >>> 2;
					k = j;
					j = c;
				}
				d[0] = d[0] + j | 0;
				d[1] = d[1] + k | 0;
				d[2] = d[2] + h | 0;
				d[3] = d[3] + g | 0;
				d[4] = d[4] + a | 0;
			},
			_doFinalize: function () {
				var b = this._data,
					f = b.words,
					d = 8 * this._nDataBytes,
					j = 8 * b.sigBytes;
				f[j >>> 5] |= 128 << 24 - j % 32;
				f[(j + 64 >>> 9 << 4) + 15] = d;
				b.sigBytes = 4 * f.length;
				this._process();
			}
		});
	g.SHA1 = i._createHelper(m);
	g.HmacSHA1 = i._createHmacHelper(m);
})();
(function () {
	var g = CryptoJS,
		i = g.enc.Utf8;
	g.algo.HMAC = g.lib.Base.extend({
		init: function (f, b) {
			f = this._hasher = f.create();
			"string" == typeof b && (b = i.parse(b));
			var g = f.blockSize,
				l = 4 * g;
			b.sigBytes > l && (b = f.finalize(b));
			for (var n = this._oKey = b.clone(), d = this._iKey = b.clone(), j = n.words, k = d.words, h = 0; h < g; h++) j[h] ^= 1549556828, k[h] ^= 909522486;
			n.sigBytes = d.sigBytes = l;
			this.reset();
		},
		reset: function () {
			var f = this._hasher;
			f.reset();
			f.update(this._iKey);
		},
		update: function (f) {
			this._hasher.update(f);
			return this;
		},
		finalize: function (f) {
			var b =
					this._hasher,
				f = b.finalize(f);
			b.reset();
			return b.finalize(this._oKey.clone()
				.concat(f));
		}
	});
})();
(function () {
	var g = CryptoJS,
		i = g.lib,
		f = i.Base,
		b = i.WordArray,
		i = g.algo,
		m = i.HMAC,
		l = i.PBKDF2 = f.extend({
			cfg: f.extend({
				keySize: 4,
				hasher: i.SHA1,
				iterations: 1
			}),
			init: function (b) {
				this.cfg = this.cfg.extend(b);
			},
			compute: function (f, d) {
				for (var g = this.cfg, k = m.create(g.hasher, f), h = b.create(), i = b.create([1]), a = h.words, e = i.words, c = g.keySize, g = g.iterations; a.length < c;) {
					var l = k.update(d)
						.finalize(i);
					k.reset();
					for (var q = l.words, t = q.length, r = l, s = 1; s < g; s++) {
						r = k.finalize(r);
						k.reset();
						for (var v = r.words, p = 0; p < t; p++) q[p] ^= v[p];
					}
					h.concat(l);
					e[0]++;
				}
				h.sigBytes = 4 * c;
				return h;
			}
		});
	g.PBKDF2 = function (b, d, f) {
		return l.create(f)
			.compute(b, d);
	};
})();


var AesUtil = function (keySize, iterationCount) {
	this.keySize = keySize / 32;
	this.iterationCount = iterationCount;
};

AesUtil.prototype.generateKey = function (salt, passPhrase) {
	var key = CryptoJS.PBKDF2(
		passPhrase,
		CryptoJS.enc.Hex.parse(salt),
		{
			keySize: this.keySize,
			iterations: this.iterationCount
		});
	return key;
};

AesUtil.prototype.encrypt = function (salt, iv, passPhrase, plainText) {
	var key = this.generateKey(salt, passPhrase);
	var encrypted = CryptoJS.AES.encrypt(
		plainText,
		key,
		{iv: CryptoJS.enc.Hex.parse(iv)});
	return encrypted.ciphertext.toString(CryptoJS.enc.Base64);
};

AesUtil.prototype.decryptWrapped = function (data, key) {

	var rawData = atob(data);
	var iv = btoa(rawData.substring(0, 16));
	var crypttext = btoa(rawData.substring(16));

	// Decrypt...
	var plaintextArray = CryptoJS.AES.decrypt(
		{
			ciphertext: CryptoJS.enc.Base64.parse(crypttext),
			salt: ""
		},
		CryptoJS.enc.Hex.parse(key),
		{iv: CryptoJS.enc.Base64.parse(iv)}
	);

	return plaintextArray.toString(CryptoJS.enc.Utf8);

};

// Convert hex string to ASCII.
// See https://stackoverflow.com/questions/11889329/word-array-to-string

function hex2a(hex) {
	var str = "";
	for (var i = 0; i < hex.length; i += 2) {
		str += String.fromCharCode(parseInt(hex.substr(i, 2), 16));
	}
	return str;
}

AesUtil.prototype.decrypt = function (salt, iv, passPhrase, cipherText) {
	var key = this.generateKey(salt, passPhrase);
	var cipherParams = CryptoJS.lib.CipherParams.create({
		ciphertext: CryptoJS.enc.Base64.parse(cipherText)
	});
	var decrypted = CryptoJS.AES.decrypt(
		cipherParams,
		key,
		{iv: CryptoJS.enc.Hex.parse(iv)});
	return decrypted.toString(CryptoJS.enc.Utf8);
}

module.exports = new AesUtil(128, 16);

});
requireRegister(M2, function (exports, require, module) {
///////////////////////////
// LIBNEO - lib.neo.utils.md5.js
// Taken from public domain facebook javascript library (md5)
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

var
	md5 = function (input) {
		// FLOW: input -> utf8 input -> bin input -> bin md5 -> utf8 md5 -> hex md5
		var
			hex_vocab = "0123456789abcdef",

			raw_input = "",
			raw_input_bits_len,
			bin_input,
			bin_md5,
			raw_md5 = "",
			hex_md5 = "",

			i = -1,
			x,
			y;


		function safe_add(x, y) {
			var lsw = (x & 0xFFFF) + (y & 0xFFFF);
			return (((x >> 16) + (y >> 16) + (lsw >> 16)) << 16) | (lsw & 0xFFFF);
		}

		// basic operations
		function cmn(q, a, b, x, s, t) {
			var num = safe_add(safe_add(a, q), safe_add(x, t));
			return safe_add((num << s) | (num >>> (32 - s)), b);
		}

		function ff(a, b, c, d, x, s, t) {
			return cmn((b & c) | ((~b) & d), a, b, x, s, t);
		}

		function gg(a, b, c, d, x, s, t) {
			return cmn((b & d) | (c & (~d)), a, b, x, s, t);
		}

		function hh(a, b, c, d, x, s, t) {
			return cmn(b ^ c ^ d, a, b, x, s, t);
		}

		function ii(a, b, c, d, x, s, t) {
			return cmn(c ^ (b | (~d)), a, b, x, s, t);
		}


		// encode string to utf-8
		while (++i < input.length) {
			// decode utf-16 surrogate pairs
			x = input.charCodeAt(i);
			y = i + 1 < input.length ? input.charCodeAt(i + 1) : 0;
			if (0xD800 <= x && x <= 0xDBFF && 0xDC00 <= y && y <= 0xDFFF) {
				x = 0x10000 + ((x & 0x03FF) << 10) + (y & 0x03FF);
				i++;
			}

			// encode raw_input as utf-8
			if (x <= 0x7F) {
				raw_input += String.fromCharCode(x);
			} else if (x <= 0x7FF) {
				raw_input += String.fromCharCode(0xC0 | ((x >>> 6) & 0x1F),
					0x80 | (x & 0x3F));
			} else if (x <= 0xFFFF) {
				raw_input += String.fromCharCode(0xE0 | ((x >>> 12) & 0x0F),
					0x80 | ((x >>> 6) & 0x3F),
					0x80 | (x & 0x3F));
			} else if (x <= 0x1FFFFF) {
				raw_input += String.fromCharCode(0xF0 | ((x >>> 18) & 0x07),
					0x80 | ((x >>> 12) & 0x3F),
					0x80 | ((x >>> 6) & 0x3F),
					0x80 | (x & 0x3F));
			}
		}


		// number of bits in the raw utf-8 string
		raw_input_bits_len = raw_input.length * 8;


		// string to little-endian array words
		bin_input = new Array(raw_input.length >> 2);
		for (i = 0; i < bin_input.length; i++) {
			bin_input[i] = 0;
		}
		for (i = 0; i < raw_input_bits_len; i += 8) {
			bin_input[i >> 5] |= (raw_input.charCodeAt(i / 8) & 0xFF) << (i % 32);
		}


		// calculate md5 as little-endian array words
		// padding
		bin_input[raw_input_bits_len >> 5] |= 0x80 << ((raw_input_bits_len) % 32);
		bin_input[(((raw_input_bits_len + 64) >>> 9) << 4) + 14] = raw_input_bits_len;

		var
			a = 1732584193,
			b = -271733879,
			c = -1732584194,
			d = 271733878;

		for (i = 0; i < bin_input.length; i += 16) {
			var
				olda = a,
				oldb = b,
				oldc = c,
				oldd = d;

			a = ff(a, b, c, d, bin_input[i], 7, -680876936);
			d = ff(d, a, b, c, bin_input[i + 1], 12, -389564586);
			c = ff(c, d, a, b, bin_input[i + 2], 17, 606105819);
			b = ff(b, c, d, a, bin_input[i + 3], 22, -1044525330);
			a = ff(a, b, c, d, bin_input[i + 4], 7, -176418897);
			d = ff(d, a, b, c, bin_input[i + 5], 12, 1200080426);
			c = ff(c, d, a, b, bin_input[i + 6], 17, -1473231341);
			b = ff(b, c, d, a, bin_input[i + 7], 22, -45705983);
			a = ff(a, b, c, d, bin_input[i + 8], 7, 1770035416);
			d = ff(d, a, b, c, bin_input[i + 9], 12, -1958414417);
			c = ff(c, d, a, b, bin_input[i + 10], 17, -42063);
			b = ff(b, c, d, a, bin_input[i + 11], 22, -1990404162);
			a = ff(a, b, c, d, bin_input[i + 12], 7, 1804603682);
			d = ff(d, a, b, c, bin_input[i + 13], 12, -40341101);
			c = ff(c, d, a, b, bin_input[i + 14], 17, -1502002290);
			b = ff(b, c, d, a, bin_input[i + 15], 22, 1236535329);

			a = gg(a, b, c, d, bin_input[i + 1], 5, -165796510);
			d = gg(d, a, b, c, bin_input[i + 6], 9, -1069501632);
			c = gg(c, d, a, b, bin_input[i + 11], 14, 643717713);
			b = gg(b, c, d, a, bin_input[i], 20, -373897302);
			a = gg(a, b, c, d, bin_input[i + 5], 5, -701558691);
			d = gg(d, a, b, c, bin_input[i + 10], 9, 38016083);
			c = gg(c, d, a, b, bin_input[i + 15], 14, -660478335);
			b = gg(b, c, d, a, bin_input[i + 4], 20, -405537848);
			a = gg(a, b, c, d, bin_input[i + 9], 5, 568446438);
			d = gg(d, a, b, c, bin_input[i + 14], 9, -1019803690);
			c = gg(c, d, a, b, bin_input[i + 3], 14, -187363961);
			b = gg(b, c, d, a, bin_input[i + 8], 20, 1163531501);
			a = gg(a, b, c, d, bin_input[i + 13], 5, -1444681467);
			d = gg(d, a, b, c, bin_input[i + 2], 9, -51403784);
			c = gg(c, d, a, b, bin_input[i + 7], 14, 1735328473);
			b = gg(b, c, d, a, bin_input[i + 12], 20, -1926607734);

			a = hh(a, b, c, d, bin_input[i + 5], 4, -378558);
			d = hh(d, a, b, c, bin_input[i + 8], 11, -2022574463);
			c = hh(c, d, a, b, bin_input[i + 11], 16, 1839030562);
			b = hh(b, c, d, a, bin_input[i + 14], 23, -35309556);
			a = hh(a, b, c, d, bin_input[i + 1], 4, -1530992060);
			d = hh(d, a, b, c, bin_input[i + 4], 11, 1272893353);
			c = hh(c, d, a, b, bin_input[i + 7], 16, -155497632);
			b = hh(b, c, d, a, bin_input[i + 10], 23, -1094730640);
			a = hh(a, b, c, d, bin_input[i + 13], 4, 681279174);
			d = hh(d, a, b, c, bin_input[i], 11, -358537222);
			c = hh(c, d, a, b, bin_input[i + 3], 16, -722521979);
			b = hh(b, c, d, a, bin_input[i + 6], 23, 76029189);
			a = hh(a, b, c, d, bin_input[i + 9], 4, -640364487);
			d = hh(d, a, b, c, bin_input[i + 12], 11, -421815835);
			c = hh(c, d, a, b, bin_input[i + 15], 16, 530742520);
			b = hh(b, c, d, a, bin_input[i + 2], 23, -995338651);

			a = ii(a, b, c, d, bin_input[i], 6, -198630844);
			d = ii(d, a, b, c, bin_input[i + 7], 10, 1126891415);
			c = ii(c, d, a, b, bin_input[i + 14], 15, -1416354905);
			b = ii(b, c, d, a, bin_input[i + 5], 21, -57434055);
			a = ii(a, b, c, d, bin_input[i + 12], 6, 1700485571);
			d = ii(d, a, b, c, bin_input[i + 3], 10, -1894986606);
			c = ii(c, d, a, b, bin_input[i + 10], 15, -1051523);
			b = ii(b, c, d, a, bin_input[i + 1], 21, -2054922799);
			a = ii(a, b, c, d, bin_input[i + 8], 6, 1873313359);
			d = ii(d, a, b, c, bin_input[i + 15], 10, -30611744);
			c = ii(c, d, a, b, bin_input[i + 6], 15, -1560198380);
			b = ii(b, c, d, a, bin_input[i + 13], 21, 1309151649);
			a = ii(a, b, c, d, bin_input[i + 4], 6, -145523070);
			d = ii(d, a, b, c, bin_input[i + 11], 10, -1120210379);
			c = ii(c, d, a, b, bin_input[i + 2], 15, 718787259);
			b = ii(b, c, d, a, bin_input[i + 9], 21, -343485551);

			a = safe_add(a, olda);
			b = safe_add(b, oldb);
			c = safe_add(c, oldc);
			d = safe_add(d, oldd);
		}
		bin_md5 = [a, b, c, d];


		// little-endian array words to a string
		for (i = 0; i < bin_md5.length * 32; i += 8) {
			raw_md5 += String.fromCharCode((bin_md5[i >> 5] >>> (i % 32)) & 0xFF);
		}


		// convert the raw md5 string to a hex md5 string
		for (i = 0; i < raw_md5.length; i++) {
			x = raw_md5.charCodeAt(i);
			hex_md5 += hex_vocab.charAt((x >>> 4) & 0x0F) + hex_vocab.charAt(x & 0x0F);
		}

		return hex_md5;
	},

	Utils_Base64 = {

		// private property
		_keyStr: "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",
		_altStr: "fuckyoUabdeghABCD89IJKLEFGH567MNOPXYZijlmTVWqrsnpQRStvwxz01234+/=",

		// public method for encoding
		encode: function (input, isAlt) {
			var output = "";
			var chr1,
				chr2,
				chr3,
				enc1,
				enc2,
				enc3,
				enc4;
			var i = 0;
			var ss = isAlt ? this._altStr : this._keyStr;

			input = this._utf8_encode(input);

			while (i < input.length) {

				chr1 = input.charCodeAt(i++);
				chr2 = input.charCodeAt(i++);
				chr3 = input.charCodeAt(i++);

				enc1 = chr1 >> 2;
				enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
				enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
				enc4 = chr3 & 63;

				if (isNaN(chr2)) {
					enc3 = enc4 = 64;
				} else if (isNaN(chr3)) {
					enc4 = 64;
				}

				output = output +
					ss.charAt(enc1) + ss.charAt(enc2) +
					ss.charAt(enc3) + ss.charAt(enc4);

			}

			return output;
		},

		// public method for decoding
		decode: function (input, isAlt) {
			var output = "";
			var chr1,
				chr2,
				chr3;
			var enc1,
				enc2,
				enc3,
				enc4;
			var i = 0;
			var ss = isAlt ? this._altStr : this._keyStr;

			input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

			while (i < input.length) {

				enc1 = ss.indexOf(input.charAt(i++));
				enc2 = ss.indexOf(input.charAt(i++));
				enc3 = ss.indexOf(input.charAt(i++));
				enc4 = ss.indexOf(input.charAt(i++));

				chr1 = (enc1 << 2) | (enc2 >> 4);
				chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
				chr3 = ((enc3 & 3) << 6) | enc4;

				output = output + String.fromCharCode(chr1);

				if (enc3 != 64) {
					output = output + String.fromCharCode(chr2);
				}
				if (enc4 != 64) {
					output = output + String.fromCharCode(chr3);
				}

			}

			output = this._utf8_decode(output);

			return output;

		},

		// private method for UTF-8 encoding
		_utf8_encode: function (string) {
			string = string.replace(/\r\n/g, "\n");
			var utftext = "";

			for (var n = 0; n < string.length; n++) {

				var c = string.charCodeAt(n);

				if (c < 128) {
					utftext += String.fromCharCode(c);
				}
				else if ((c > 127) && (c < 2048)) {
					utftext += String.fromCharCode((c >> 6) | 192);
					utftext += String.fromCharCode((c & 63) | 128);
				}
				else {
					utftext += String.fromCharCode((c >> 12) | 224);
					utftext += String.fromCharCode(((c >> 6) & 63) | 128);
					utftext += String.fromCharCode((c & 63) | 128);
				}

			}

			return utftext;
		},

		// private method for UTF-8 decoding
		_utf8_decode: function (utftext) {
			var string = "";
			var i = 0,
				c2,
				c3;
			var c = c2 = 0;

			while (i < utftext.length) {

				c = utftext.charCodeAt(i);

				if (c < 128) {
					string += String.fromCharCode(c);
					i++;
				}
				else if ((c > 191) && (c < 224)) {
					c2 = utftext.charCodeAt(i + 1);
					string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
					i += 2;
				}
				else {
					c2 = utftext.charCodeAt(i + 1);
					c3 = utftext.charCodeAt(i + 2);
					string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
					i += 3;
				}

			}

			return string;
		}

	},
	Utils_Base64Binary = {
		_keyStr: "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

		/* will return a  Uint8Array type */
		decodeArrayBuffer: function (input) {
			var bytes = Math.ceil((3 * input.length) / 4.0);
			var ab = new window.ArrayBuffer(bytes);
			this.decode(input, ab);

			return ab;
		},

		decode: function (input, arrayBuffer) {
			//get last chars to see if are valid
			var lkey1 = this._keyStr.indexOf(input.charAt(input.length - 1));
			var lkey2 = this._keyStr.indexOf(input.charAt(input.length - 1));

			var bytes = Math.ceil((3 * input.length) / 4.0);
			if (lkey1 == 64) bytes--; //padding chars, so skip
			if (lkey2 == 64) bytes--; //padding chars, so skip

			var uarray;
			var chr1,
				chr2,
				chr3;
			var enc1,
				enc2,
				enc3,
				enc4;
			var i,
				j = 0;

			if (arrayBuffer) {
				uarray = new window.Uint8Array(arrayBuffer);
			} else {
				uarray = new window.Uint8Array(bytes);
			}

			input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

			for (i = 0; i < bytes; i += 3) {
				//get the 3 octects in 4 ascii chars
				enc1 = this._keyStr.indexOf(input.charAt(j++));
				enc2 = this._keyStr.indexOf(input.charAt(j++));
				enc3 = this._keyStr.indexOf(input.charAt(j++));
				enc4 = this._keyStr.indexOf(input.charAt(j++));

				chr1 = (enc1 << 2) | (enc2 >> 4);
				chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
				chr3 = ((enc3 & 3) << 6) | enc4;

				uarray[i] = chr1;
				if (enc3 != 64) uarray[i + 1] = chr2;
				if (enc4 != 64) uarray[i + 2] = chr3;
			}

			return uarray;
		},
		encodeArrayBuffer: function (arrayBuffer) {

			var base64 = "";
			var encodings = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

			var bytes = new Uint8Array(arrayBuffer);
			var byteLength = bytes.byteLength;
			var byteRemainder = byteLength % 3;
			var mainLength = byteLength - byteRemainder;

			var a,
				b,
				c,
				d;
			var chunk;

			// Main loop deals with bytes in chunks of 3
			for (var i = 0; i < mainLength; i = i + 3) {
				// Combine the three bytes into a single integer
				chunk = (bytes[i] << 16) | (bytes[i + 1] << 8) | bytes[i + 2];

				// Use bitmasks to extract 6-bit segments from the triplet
				a = (chunk & 16515072) >> 18; // 16515072 = (2^6 - 1) << 18
				b = (chunk & 258048) >> 12; // 258048   = (2^6 - 1) << 12
				c = (chunk & 4032) >> 6; // 4032     = (2^6 - 1) << 6
				d = chunk & 63;               // 63       = 2^6 - 1

				// Convert the raw binary segments to the appropriate ASCII encoding
				base64 += encodings[a] + encodings[b] + encodings[c] + encodings[d];
			}

			// Deal with the remaining bytes and padding
			if (byteRemainder == 1) {
				chunk = bytes[mainLength];

				a = (chunk & 252) >> 2; // 252 = (2^6 - 1) << 2

				// Set the 4 least significant bits to zero
				b = (chunk & 3) << 4; // 3   = 2^2 - 1

				base64 += encodings[a] + encodings[b] + "==";
			} else if (byteRemainder == 2) {
				chunk = (bytes[mainLength] << 8) | bytes[mainLength + 1];

				a = (chunk & 64512) >> 10; // 64512 = (2^6 - 1) << 10
				b = (chunk & 1008) >> 4; // 1008  = (2^6 - 1) << 4

				// Set the 2 least significant bits to zero
				c = (chunk & 15) << 2; // 15    = 2^4 - 1

				base64 += encodings[a] + encodings[b] + encodings[c] + "=";
			}

			return base64;
		}
	};

module.exports = md5;


});
requireRegister(M3, function (exports, require, module) {
/* global ERROR,  PROMISE */

const
	S3ChannelPromises = require(M5);

const
	DEFAULT_REGION = "eu-west-1";

let
	mS3ChannelClient = null;

function init() {
	return new PROMISE((resolve, reject) => {
		if (mS3ChannelClient) {
			resolve();
		} else {
			try {
				const NobjAuth = require("nobj.auth");
				mS3ChannelClient = new S3ChannelPromises(NobjAuth.external("sbck"), NobjAuth.channel.info().id, NobjAuth.external("sreg", DEFAULT_REGION));
				resolve();
			} catch (e) {
				reject(e);
			}
		}
	});
}

module.exports = {
	init,
	client: () => {
		if (mS3ChannelClient) return mS3ChannelClient;
		throw new ERROR("Not inited");
	},
	BUCKET_META_CDNROOT: "meta",
	BUCKET_PUBLIC_CDNROOT: "public",
	BUCKET_INCOMING_CDNROOT: "incoming"
};

});
requireRegister(M4, function (exports, require, module) {
/* eslint-disable no-nested-ternary,max-len */
/* global D,ObjectAssign,PROMISE */

const

	NobjAuth = require("nobj.auth"),
	AES = require(M1),
	md5 = require(M2),

	TAG = "[S3]",
	DEFAULT_MIME = "text/plain";

function loadScript(url, callback, doc) {

	return new PROMISE((resolve, reject) => {
		const
			head = (doc || document).getElementsByTagName("head")[0],
			script = (doc || document).createElement("script");

		ObjectAssign(script, {
			type: "text/javascript",
			charset: "UTF-8",
			src: url,
			onerror: () => {
				reject(new Error("loading error"));
			}
		});

		// noinspection JSUndefinedPropertyAssignment
		script.onload = script.onreadystatechange = function () {
			if (!this.readyState || this.readyState === "loaded" || this.readyState === "complete") {
				// script successfully loaded
				resolve();
			} else {
				reject(new Error("loading error"));
			}
		};

		script.onerror = () => {
			reject(new Error("loading error"));
		};

		head.appendChild(script);
	});
}

function initSdk() {

	return new PROMISE((resolve, reject) => {

		if (window.AWS) {
			resolve(window.AWS);
		} else if (!window.AWS_LOADING) {
			window.AWS_LOADING = true;
			loadScript("https://sdk.amazonaws.com/js/aws-sdk-2.263.1.min.js")
				.then(() => {
					window.AWS_LOADING = false;
					resolve(window.AWS);
				})
				.catch((error) => {
					window.AWS_LOADING = false;
					reject(error);
				});
		} else {

			const waitFor = () => {
				if (!window.AWS) {
					setTimeout(waitFor, 500);
				} else {
					resolve(window.AWS);
				}
			};

			// two simultaneous calls
			setTimeout(waitFor, 500);
		}

	});

}

/**
 * S3 client
 * @param {string} s3bucket The S3 bucket
 * @param {string} cdnroot The CDN Root (public / meta)
 * @param {string} channelname Channel Name
 * @param {string=} region AWS Region
 * @constructor
 */

function S3ChannelClient(s3bucket, cdnroot, channelname, region) {

	let mClient = null;

	function getPolicy() {
		const
			rawPolicy = NobjAuth.external("spol"),
			jsonPolicy = AES.decryptWrapped(rawPolicy, md5(channelname));
		return JSON.parse(jsonPolicy);
	}

	function initClient(bucket) {

		return new PROMISE((resolve, reject) => {

			initSdk()
				.then((AWS) => {
					try {
						const policy = getPolicy();

						if (!policy) {
							reject(new Error("no policy"));
						} else if (mClient) {
							resolve(mClient);
						} else {

							// noinspection JSUnresolvedVariable
							AWS.config.update({
								credentials: {
									expireTime: policy.Expiration,
									accessKeyId: policy.AccessKeyId,
									secretAccessKey: policy.SecretAccessKey,
									sessionToken: policy.SessionToken
								}
							});

							mClient = new AWS.S3({
								region,
								signatureVersion: "v4",
								params: {
									Bucket: bucket
								}
							});
							resolve(mClient);

						}
					} catch (e) {
						reject(e);
					}
				})
				.catch(reject);
		});
	}

	function getRoot(key, overrideChannel) {

		return (
			cdnroot
				? [cdnroot, overrideChannel || channelname, key]
				: [key] // [overrideChannel || channelname, key]
		).join("/");
	}

	function list(path, callback, overrideChan) {

		return new PROMISE((resolve, reject) => {
			initClient(s3bucket)
				.then((client) => {
					// noinspection JSUnresolvedFunction
					client.listObjects(
						{
							Prefix: getRoot(path ? path + "/" : "", overrideChan),
							Delimiter: "/"
						},
						(error, data) => {
							if (error) {
								reject(error);
							} else {
								resolve(data);
							}
						}
					);
				})
				.catch(reject);
		});
	}

	function listRaw(entityType) {

		return new PROMISE((resolve, reject) => {

			initClient(s3bucket)
				.then((client) => {
					client.listObjects(
						{
							Prefix: getRoot(entityType ? entityType + "/" : "", null)
						},
						(error, data) => {
							if (error) {
								reject(error);
							} else {
								resolve(data);
							}
						}
					);
				});
		});
	}

	function getSignedUrl(key, overrideChannel) {
		return new PROMISE((resolve, reject) => {
			initClient(s3bucket)
				.then((client) => {

					client.getSignedUrl(
						"getObject",
						{
							Key: getRoot(key, overrideChannel)
						},
						(error, data) => {
							if (error) {
								reject(error);
							} else {
								resolve(data);
							}
						}
					);
				});
		});
	}

	function jsonParse(jsonp) {

		jsonp = jsonp.trim();

		const
			jtok = jsonp.indexOf("{"),
			jtok2 = jsonp.indexOf("["),
			ptok = jsonp.indexOf("(");

		if (ptok > 0 && (ptok < jtok && ptok < jtok2)) {
			jsonp = jsonp.substring(ptok + 1);
			jsonp = jsonp.substring(0, jsonp.lastIndexOf(")"));
		}

		return JSON.parse(jsonp);

	}

	function del(key, options) {

		options = options || {};

		return new PROMISE((resolve, reject) => {

			initClient(s3bucket)
				.then((client) => {
					const params = {
						Key: getRoot(key, options.overrideChan)
					};
					// noinspection JSUnresolvedFunction
					client.deleteObject(params, (err, data) => {
						if (err) {
							reject(err);
						} else {
							resolve(data);
						}
					});
				})
				.catch(reject);
		});
	}

	function get(key, options) {

		options = options || {};

		return new PROMISE((resolve, reject) => {

			initClient(s3bucket)
				.then((client) => {
					const params = {
						Key: getRoot(key, options.overrideChan)
					};
					// noinspection JSUnresolvedFunction
					client.getObject(params, (err, data) => {
						window.console.log("got  " + data);
						if (err) {
							reject(err);
						} else {
							try {
								const
									str = data.Body ? data.Body.toString() : null,
									jdata = str ? (options.raw ? str : jsonParse(str)) : null;

								if (jdata != null) {
									resolve(jdata);
								} else {
									reject(new Error("no file found"));
								}

							} catch (e) {
								reject(e);
							}
						}
					});
				})
				.catch(reject);

		});

	}

	/*


	var params = {

	    Bucket: 'STRING_VALUE', / required /
		Key: 'STRING_VALUE', / required /

		ACL: private | public-read | public-read-write | authenticated-read | aws-exec-read | bucket-owner-read | bucket-owner-full-control,
		Body: new Buffer('...') || 'STRING_VALUE' || streamObject,
		CacheControl: 'STRING_VALUE',
		ContentDisposition: 'STRING_VALUE',
		ContentEncoding: 'STRING_VALUE',
		ContentLanguage: 'STRING_VALUE',
		ContentLength: 0,
		ContentMD5: 'STRING_VALUE',
		ContentType: 'STRING_VALUE',
		Expires: new Date || 'Wed Dec 31 1969 16:00:00 GMT-0800 (PST)' || 123456789,
		GrantFullControl: 'STRING_VALUE',
		GrantRead: 'STRING_VALUE',
		GrantReadACP: 'STRING_VALUE',
		GrantWriteACP: 'STRING_VALUE',
		Metadata: {
			'<MetadataKey>': 'STRING_VALUE', / '<MetadataKey>': ... /
		},
		RequestPayer: requester,
		SSECustomerAlgorithm: 'STRING_VALUE',
		SSECustomerKey: new Buffer('...') || 'STRING_VALUE' // Strings will be Base-64 encoded on your behalf /,
		SSECustomerKeyMD5: 'STRING_VALUE',
		SSEKMSKeyId: 'STRING_VALUE',
		ServerSideEncryption: AES256 | aws:kms,
		StorageClass: STANDARD | REDUCED_REDUNDANCY | STANDARD_IA,
		Tagging: 'STRING_VALUE',
		WebsiteRedirectLocation: 'STRING_VALUE'
	};
	*/


	function getMimeFor(key, contents) {

		if (typeof contents === "string") {
			if (key.endsWith(".png")) {
				return "image/png";
			}
			if (key.endsWith(".jpg")) {
				return "image/jpeg";
			}
			if (key.endsWith(".jpeg")) {
				return "image/jpeg";
			}
			if (key.endsWith(".webm")) {
				return "image/webm";
			}
			if (key.endsWith(".mp4")) {
				return "video/mp4";
			}
			if (key.endsWith(".json")) {
				return "application/json";
			}
			if (key.endsWith(".html")) {
				return "text/html";
			}
			if (key.endsWith(".js")) {
				return "application/javascript";
			}
			return "text/plain";
		}

		if (contents.type) {
			return contents.type;
		}

		return DEFAULT_MIME;
	}

	function put(key, contents, options) {

		options = options || {};

		return new PROMISE((resolve, reject) => {


			const
				params = {
					Body: contents,
					Key: getRoot(key),
					ContentType: options.mime || getMimeFor(key, contents),
					...options.encoding && {ContentEncoding: options.encoding},
					...options.attachment && {ContentDisposition: "attachment"}
				};

			if (options) {
				for (const o of Object.keys(options)) {
					if (o === "meta") {

						// meta parameter are file headers x-amz-meta

						const ometa = options[o];
						for (const metakey of Object.keys(ometa)) {
							params["x-amz-meta-" + metakey] = ometa[metakey];
						}

					} else if (o !== "mime" && o !== "encoding" && o !== "validator" && o !== "attachment") {

						// standard parameter i PutObject parameter
						// http://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectPUT.html

						params[o] = options[o];

					}

				}
			}

			initClient(s3bucket)
				.then((client) => {
					// noinspection JSUnresolvedFunction
					client.putObject(params, (err, data) => {
						if (err) {
							reject(err);
						} else {
							resolve(data);
						}
					});
				});
		});
	}


	function setMeta(key, metadata) {

		return new PROMISE((resolve, reject) => {
			initClient(s3bucket)
				.then((client) => {

					// noinspection JSUnresolvedFunction
					client.headObject(
						{
							Bucket: s3bucket,
							Key: getRoot(key)
						},
						(err, headData) => {
							if (err) {
								if (D) window.console.error("* Cant complete .headObject, error details are: ");
								reject(err);
							} else {

								window.console.log("- lets update", key, "(", headData.ContentType, ")", " to have the new cache control headers...");
								if (metadata) {
									for (const mkey of Object.keys(metadata)) {
										headData["x-amz-meta-" + mkey] = metadata[mkey];
									}
								}

								// noinspection JSUnresolvedFunction
								client.copyObject(headData, (error) => {
									if (error) {
										if (D) console.error(TAG, "* Cant complete .copyObject for", key, ", error details are: ");
										reject(error);
									} else {
										if (D) console.error(TAG, "- copyObject done and headers should be updated for :", key);
										resolve(true);
									}
								});
							}
						}
					);
				})
				.catch(reject);
		});
	}


	// noinspection JSUnusedGlobalSymbols
	return {
		list,
		listRaw,
		get,
		put,
		del,
		getSignedUrl,
		setMeta,
		getMimeFor
	};
}


/**
 * S3 client
 * @type {function(string, string, string, string, boolean): {list: (function(*=, *, *=)), listRaw: (function(*=)), get: (function(*=, *=)), put: (function(*=, *=, *=)), putImage: (function(*=, *=, *)), getSignedUrl: (function(*=, *, *=)), setMeta: (function(*=, *=))}}
 */

module.exports = S3ChannelClient;

});
requireRegister(M5, function (exports, require, module) {
/* global PROMISE, ObjectAssign */

const
	S3Channel = require(M4);

const
	BUCKET_META_CDNROOT = "meta",
	BUCKET_PUBLIC_CDNROOT = "public",
	BUCKET_INCOMING_CDNROOT = "incoming",

	MIMES = {
		jpg: {
			bin: true,
			mime: "image/jpeg"
		},
		jpeg: {
			bin: true,
			mime: "image/jpeg"
		},
		gif: {
			bin: true,
			mime: "image/gif"
		},
		png: {
			bin: true,
			mime: "image/png"
		},
		svg: {
			bin: true,
			mime: "image/svg+xml"
		},
		webm: {
			bin: true,
			mime: "image/webm"
		},
		txt: {
			bin: false,
			mime: "text/plain"
		},
		js: {
			bin: false,
			mime: "application/javascript"
		},
		json: {
			bin: false,
			mime: "application/json"
		},
		mp4: {
			bin: true,
			mime: "video/mp4"
		},
		mp3: {
			bin: true,
			mime: "audio/mp3"
		},
		wav: {
			bin: true,
			mime: "audio/wav"
		}
	};

function S3ChannelClient(appName, channelName, region) {

	function getBucket() {
		return appName;
	}

	const
		mS3Public = new S3Channel(getBucket(), BUCKET_PUBLIC_CDNROOT, channelName, region),
		mS3Incoming = new S3Channel(getBucket(), BUCKET_INCOMING_CDNROOT, channelName, region),
		mS3Root = new S3Channel(getBucket(), null, channelName, region),
		mS3Meta = new S3Channel(getBucket(), BUCKET_META_CDNROOT, channelName, region);

	return {

		get: (cdnroot) => {
			switch (cdnroot) {
				case BUCKET_PUBLIC_CDNROOT:
					return mS3Public;
				case BUCKET_INCOMING_CDNROOT:
					return mS3Incoming;
				case BUCKET_META_CDNROOT:
					return mS3Meta;
				default:
					return mS3Root;
			}
		}
	};
}

function S3ChannelPromises(appName, channelName, region) {

	const s3channelclient = new S3ChannelClient(appName, channelName, region);

	function getClient(cdnroot) {
		return PROMISE.resolve(s3channelclient.get(cdnroot));
	}

	function put(key, cdnroot, data, options) {
		return new PROMISE((resolve, reject) => {
			getClient(cdnroot)
				.then((client) => {
					client.put(
						key,
						data,
						options
					)
						.then(resolve)
						.catch(reject);
				})
				.catch(reject);
		});
	}

	function del(key, cdnroot, options) {
		return new PROMISE((resolve, reject) => {
			getClient(cdnroot)
				.then((client) => {
					client.del(
						key,
						options
					)
						.then(resolve)
						.catch(reject);
				})
				.catch(reject);
		});
	}

	/*
	function s3api_publishEntity(type, data, splash) {
		if (type && data.syspath) {
			NApiFireOwn.entity.tag(type, data);
			var path = data.type + "/" + data.syspath + "/content.json";
			return saveJson(path, BUCKET_PUBLIC_CDNROOT, data, splash);
		} else {
			throw new Error("No entity type or syspath");
		}
	}
	function s3api_publishPublicData(prefix, type, payload, splashFunction) {

		const name = payload.name;
		if (name) {
			return save(
				(prefix || "") + "data/" + (type ? type + "/" : "") + name + ".json",
				BUCKET_PUBLIC_CDNROOT,
				JSON.stringify(payload),
				splashFunction
			);
		}

	}
	*/

	function putJson(key, cdnroot, data) {

		return new PROMISE((resolve, reject) => {
			if (!data || !cdnroot || !key) {
				reject(new Error("invalid parameters"));
			} else {
				getClient(cdnroot)
					.then((client) => {
						client.put(key, JSON.stringify(data), {attachment: true})
							.then(resolve)
							.catch(reject);
					})
					.catch(reject);
			}
		});
	}

	function get(key, cdnroot, options) {

		return new PROMISE((resolve, reject) => {
			if (!key) {
				reject(new Error("invalid parameters"));
			} else {
				getClient(cdnroot || null)
					.then((client) => {
						client.get(key, options)
							.then(resolve)
							.catch(reject);
					})
					.catch(reject);
			}
		});
	}

	function getSignedUrl(key, cdnroot, options) {

		options = options || {};
		return new PROMISE((resolve, reject) => {
			if (!key) {
				reject(new Error("invalid parameters"));
			} else {
				getClient(cdnroot || null)
					.then((client) => {
						client.getSignedUrl(key, options.overrideChan)
							.then(resolve)
							.catch(reject);
					})
					.catch(reject);
			}
		});
	}


	function getExtension(filename) {
		return filename.substring(filename.lastIndexOf(".") + 1)
			.toLowerCase();
	}

	function getMimeFor(filename, def) {
		const ext = getExtension(filename);
		return MIMES[ext].mime || def || "text/plain";
	}

	function isBinary(filename) {
		const ext = getExtension(filename);
		return MIMES[ext].bin;
	}

	function getItemIcon(filename) {

		switch (getExtension(filename)) {
			case "jpg":
			case "jpeg":
			case "webm":
			case "gif":
			case "png":
				return "fa-file-image";
			case "txt":
				return "fa-file-alt";
			case "pdf":
				return "fa-file-pdf";
			case "mp4":
				return "fa-file-video";
			case "json":
			case "js":
				return "file-code";
			default:
				return "file";
		}
	}

	function getMimeInfo(name) {
		return {
			icon: getItemIcon(name),
			ext: getExtension(name),
			mime: getMimeFor(name)
		};
	}

	function uploadFile(key, cdnroot, file, options) {

		options = options || {};

		return new Promise((resolve, reject) => {

			const reader = new FileReader();

			reader.onload = function (evt) {

				const
					data = evt.target.result;

				if (data) {

					options.mime = options.mime || getMimeFor(file.name);

					if (options.validator && typeof options.validator === "function") {
						try {
							options.validator({
								name: file.name,
								info: getMimeInfo(file.name),
								contents: data
							});
						} catch (e) {
							reject(e);
							return;
						}
					}

					put(key, cdnroot, data, options)
						.then(resolve)
						.catch(reject);

				} else {
					reject(new Error("empty data"));
				}
			};

			if (options.dataUrl) {
				reader.readAsDataURL(file);
			} else if (isBinary(file.name)) {
				reader.readAsArrayBuffer(file);
			} else {
				reader.readAsText(file);
			}
		});
	}

	function createItemFolder(cdnroot, item) {

		const prefix = item.Prefix,
			paths = prefix.split("/"),
			name = paths[paths.length - 2];

		return {
			name,
			type: "folder",
			title: name,
			Prefix: prefix,
			icon: "folder",
			cdnroot
		};

	}

	function createItemFile(cdnroot, item) {

		const
			itemKey = item.Key,
			paths = itemKey.split("/"),
			name = paths[paths.length - 1];

		if (!name) return null;

		ObjectAssign(item, {
			name,
			type: "file",
			title: name.indexOf(".") !== -1 ? name.substring(0, name.lastIndexOf(".")) : name,
			mime: getMimeFor(name),
			cdnroot,
			icon: getItemIcon(name),
			updated: item.LastModified.valueOf()
		});

		return item;

	}

	function list(key, cdnroot, parser) {

		return new PROMISE((resolve, reject) => {

			getClient(cdnroot)
				.then((client) => {
					client.list(key)
						.then((data) => {

							const arrayItems = [];

							let folder = null,
								item = null;

							if (data) {
								// folders
								for (let i = 0, l = data.CommonPrefixes.length; i < l; i++) {

									folder = data.CommonPrefixes[i];
									item = createItemFolder(cdnroot, folder);

									arrayItems.push(item);
								}

								// files
								for (let i = 0, l = data.Contents.length; i < l; i++) {
									folder = data.Contents[i];
									item = createItemFile(cdnroot, folder);
									if (item) arrayItems.push(item);
								}
							}

							resolve(parser ? arrayItems.map(parser) : {data: arrayItems});
						})
						.catch(reject);
				})
				.catch(reject);

		});

	}

	// noinspection JSUnusedGlobalSymbols
	return {
		list,
		put,
		putJson,
		getSignedUrl,
		del,
		get: (key, cdnroot) => get(key, cdnroot, {raw: true}),
		getJson: (key, cdnroot) => get(key, cdnroot, {raw: false}),
		getMimeFor,
		getMimeInfo,
		uploadFile
	};
}

module.exports = S3ChannelPromises;

});

;

})(
    Promise,
    Object.assign,
    Object.keys,
    window.setTimeout,
    window.clearTimeout,
    window.location,
    window.addEventListener,
    document,
    document.body,
    document.documentElement,
    document.documentElement.classList,
    window,
    window.NIPC || null,
    Error
);
})("131b5289ac9efcbc3eb0ffa56aac71b6","6d3c5993ca017d0ff169b425d3193f02","1bc29b36f623ba82aaf6724fd3b16718","a7297328fa313643e3213faae4be82e8","f800144708a7add8ad16b7a89612fbbb","5cc0eda8534c8a3acef5f6ca6138ed0c",0)
